#include "AmaClientBoard.H"
#include "AmaClientMove.H"
#include <vector>
#include <algorithm> 

void AmaClientBoard::setup()
{
  clear();

  sint4 w = width();
  sint4 u = w/2-2;
  
  sq[ClientBoard::xy2ind(0, u)]     = BLACK;
  sq[ClientBoard::xy2ind(u, 0)]     = BLACK;
  sq[ClientBoard::xy2ind(w-u-1, 0)] = BLACK;
  sq[ClientBoard::xy2ind(w-1, u)]   = BLACK;
  
  sq[ClientBoard::xy2ind(0, w-u-1)]   = WHITE;
  sq[ClientBoard::xy2ind(u, w-1)]     = WHITE;
  sq[ClientBoard::xy2ind(w-u-1, w-1)] = WHITE;
  sq[ClientBoard::xy2ind(w-1, w-u-1)] = WHITE;
  
  turn_color = ClientColor::BLACK;
}

bool AmaClientBoard::is_finished() const
{
  return false;
}


bool AmaClientBoard::on_same_line(sint4 ind1, sint4 ind2) const
{
  sint4 x1, y1, x2, y2;
  
  x1 = ClientBoard::ind2x(ind1);
  y1 = ClientBoard::ind2y(ind1);
  x2 = ClientBoard::ind2x(ind2);
  y2 = ClientBoard::ind2y(ind2);
  
  return x1 == x2 || y1 == y2 || abs(x1-x2) == abs(y1-y2);
}

static sint4 sign(sint4 d)
{
  if (d > 0) return 1;
  if (d < 0) return -1;
  return 0;
}

bool AmaClientBoard::empty_line(sint4 ind1, sint4 ind2) const
{
  if (ind1 == ind2) return false;
  if (!on_same_line(ind1, ind2)) return false;
  sint4 x1, y1, x2, y2;

  x1 = ClientBoard::ind2x(ind1);
  y1 = ClientBoard::ind2y(ind1);
  x2 = ClientBoard::ind2x(ind2);
  y2 = ClientBoard::ind2y(ind2);

  sint4 dx = sign(x2-x1);
  sint4 dy = sign(y2-y1);
  sint4 d = dx + dy * ClientBoard::DX;
  sint4 p = ind1 + d;

  while (p != ind2) {
    if (sq[p] != EMPTY) return false;
    p += d;
  }
  
  return true;
}

sint4 AmaClientBoard::char2cont(char c) const
{
  switch (c) {
  case '*': return BLACK;
  case 'O': return WHITE;
  case '-': return EMPTY;
  case '+': return HIT;
  default:
    ERR("illegal cont. char.");
  }
  return EMPTY;
}

sint1 AmaClientBoard::cont2char(sint4 co) const
{
  switch (co) {
  case BLACK:  return '*';
  case WHITE:  return 'O';
  case EMPTY:  return '-';
  case HIT:    return '+';    
  case BORDER: return '#';
  default:
    ERR("illegal cont.");
  }
  return '?';
}


// slow

bool AmaClientBoard::no_opponent_moves()
{
  sint4 check_color = ClientColor::opponent(turn_color);
    
  FORS (i, MAX_BOARD_SIZE) {
    if (sq[i] == check_color) {
      FORS (j, 8) {
        if (sq[i+d[j]] == EMPTY) return false;
      }
    }
  }

  return true;
}


// true iff move ok

bool AmaClientBoard::make_move(const ClientMove &mv0)
{
  const AmaClientMove &mv = dynamic_cast<const AmaClientMove&>(mv0);
  
  if (!mv.is_valid()) return false;
  
  int from, to, arrow;

  from  = mv.get_from();
  to    = mv.get_to();
  arrow = mv.get_arrow();
  
  if (sq[from] != turn_color) return false;
  if (sq[to]   != EMPTY)      return false;
  if (from != arrow &&
      sq[arrow] != EMPTY)     return false;    
    
  if (!on_same_line(from, to))  return false;
  if (!on_same_line(to, arrow)) return false;  
  if (!empty_line(from, to))    return false;

  sq[to]   = turn_color;
  sq[from] = EMPTY;

  if (!empty_line(to, arrow)) {
    sq[to] = EMPTY;
    sq[from] = turn_color;
    return false;
  }

  sq[arrow] = HIT;

  if (no_opponent_moves()) {
    white_passes += turn_color == BLACK ? 1 : -1;
  } else toggle_to_move();

  return true;
}


#if 0
void AmaClientBoard::write(ostream &os, sint4 protocol) const
{
  sint4 w = type.board_width(), x, y;
  
  switch (protocol) {

  case 0:
    //os << "type: ";
    //type.write(os);
    //os << EOL;
    os << "  ";
    FOR (x, w) os.form(" %c", 'A'+x);
    os << endl;
      
    FOR (y, w) {
      os.form("%2d", y+1);
      FOR (x, w) {
	ClientBoard::SqCont co = squares[ClientBoard::xy2ind(x, y)];
	if      (co == BLACK) os << " *";
	else if (co == WHITE) os << " O";
	else if (co == EMPTY) os << " -";
	else                  os << "  ";
      }
      os.form(" %-2d", y+1);
      os << endl;
    }

    os << "  ";
    FOR (x, w) os.form(" %c", 'A'+x);
    os << endl << endl;
    if (get_to_move() == ClientColor::BLACK) os << "* to move"; else os << "O to move";
    os << endl;
    break;
  
  case 1:
    FOR (y, w) {
      FOR (x, w) {
	ClientBoard::SqCont co = squares[ClientBoard::xy2ind(x, y)];
	if      (co == BLACK) os << "*";
	else if (co == WHITE) os << "O";
	else if (co == EMPTY) os << "-";
	else                                os << " ";
      }
      os << endl;
    }
    if (get_to_move() == ClientColor::BLACK) os << "* to move"; else os << "O to move";
    os << endl;
    break;

  default: ERR("unknown protocol");
  }

}
#endif

