#include "AmaClientMove.H"
#include "ClientBoard.H"

// parse ascii-move [e.g. "E13" ]
// return true iff syntax correct

String AmaClientMove::str2coords(String &s)
{
  if (s == "") return "empty move string";
  
  vector<String> vec;
  String::parse(s, vec, '-');

  if (vec.size() != 3) {
    return "illegal coords; e.g. a1-a8-b8";
  }
    
  sint4 coords[3];

  FORS (i, 3) {

    sint4 x = toupper(vec[i][0]) - 'A';
    if (x < 0 || x >= ClientBoard::MAX_BOARD_WIDTH) {
      return "sq-x out of range";
    }
    
    const char *p = vec[i].c_str();
    sint4 y;
    
    if (sscanf(&p[1], "%d", &y) != 1) return "missing sq-y";
    
    --y;
    
    if (y < 0 || y >= ClientBoard::MAX_BOARD_WIDTH) {
      return "sq-y out of range";
    }

    coords[i] = ClientBoard::xy2ind(x, y);
  }

  init(coords[0], coords[1], coords[2]);
  return "";
}


bool AmaClientMove::is_valid() const
{
  return
    from >= 0  && from  < ClientBoard::MAX_BOARD_SIZE &&
    to >= 0    && to    < ClientBoard::MAX_BOARD_SIZE &&
    arrow >= 0 && arrow < ClientBoard::MAX_BOARD_SIZE;
}

String AmaClientMove::coords2str() const
{
  if (from == UNDEF) {

    return "?";

  } else {

    return coord2str(from) + "-" + coord2str(to) + "-" + coord2str(arrow);
  }

}

bool comp(const AmaClientMove &mv1, const AmaClientMove &mv2)
{
  if (mv1.get_eval() < mv2.get_eval()) return true;
  if (mv1.get_eval() > mv2.get_eval()) return false;
  
  if (mv1.get_from() < mv2.get_from()) return true;
  if (mv1.get_from() > mv2.get_from()) return false;
  
  if (mv1.get_to() < mv2.get_to()) return true;
  if (mv1.get_to() > mv2.get_to()) return false;
  
  if (mv1.get_arrow() < mv2.get_arrow()) return true;
  if (mv1.get_arrow() > mv2.get_arrow()) return false;

  return false;
}

