/*
    (c) Michael Buro

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "AxClientBoard.H"
#include "AxClientMove.H"
#include <vector>
#include <algorithm> 


void AxClientBoard::setup()
{
  clear();
}

#if 0
bool AxClientBoard::is_finished() const
{
  if (!no_moves(false)) return false;
  return no_moves(true);
}
#endif

sint4 AxClientBoard::char2cont(char c) const
{
  switch (c) {
  case '*': return BLACK;
  case 'O': return WHITE;
  case '-': return EMPTY;
  case '+': return BLOCKED;
  default:
    ERR("illegal char in board.sgf");
  }
  return EMPTY;
}

sint1 AxClientBoard::cont2char(sint4 co) const
{
  switch (co) {
  case BLACK:  return '*';
  case WHITE:  return 'O';
  case EMPTY:  return '-';
  case BLOCKED: return '+';    

  default:
    ERR("illegal cont");
  }
  return '?';
}

#if 0

bool AxClientBoard::no_moves(bool opponent) const
{
  AxClientBoard bo = *this;
  
  if (opponent) bo.toggle_to_move();

  FORS (i, ClientBoard::MAX_BOARD_SIZE) {
    AxClientMove mv(i);
    if (bo.make_move(mv)) return false;
  }

  return true;
}


int AxClientBoard::disc_num(sint4 cont) const
{
  int n = 0;
  
  FORS (i, ClientBoard::MAX_BOARD_SIZE) {
    if (sq[i] == cont) n++;
  }

  return n;
}

int AxClientBoard::disc_num() const
{
  int n = 0;
  
  FORS (i, ClientBoard::MAX_BOARD_SIZE) {
    if (sq[i] == BLACK || sq[i] == WHITE) n++;
  }

  return n;
}

#endif


// true iff move ok

bool AxClientBoard::make_move(const ClientMove &mv0)
{
  const AxClientMove &mv = dynamic_cast<const AxClientMove&>(mv0);

  assert(mv.is_valid());

  sint4 turn_cont, opp_cont;
  
  if (turn_color == ClientColor::BLACK) { turn_cont = BLACK; opp_cont = WHITE; }
  else                                  { turn_cont = WHITE; opp_cont = BLACK; }

  if (sq[mv.from] != turn_cont) return false;
  if (sq[mv.to]   != EMPTY) return false;  
  
  sq[mv.to] = turn_cont;
  
  sint4 x0,y0,x1,y1;

  x0 = ind2x(mv.from);
  y0 = ind2y(mv.from); 
  x1 = ind2x(mv.to);
  y1 = ind2y(mv.to); 
 
  sint4 absx = abs(x0-x1), absy = abs(y0-y1);

  if (absx >= 2 || absy >= 2) {

    // jumping

    sq[mv.from] = EMPTY;
  }

  // flip opponent's discs in vicinity

  FORS (i, 8) {
    if (sq[mv.to+d[i]] == opp_cont)
      sq[mv.to+d[i]] = turn_cont;
  }

  if (!no_moves(true)) turn_color = ClientColor::opponent(turn_color);
 
  return true;
}


bool AxClientBoard::no_moves(bool opponent) const
{
  sint4 turn_cont;
  
  if (turn_color == ClientColor::BLACK ^ opponent) {
    turn_cont = BLACK;
  } else {
    turn_cont = WHITE;
  }

  int i;
  FOR (i, MAX_BOARD_SIZE) {
    if (sq[i] == turn_cont) {
      for (sint4 dx = -2; dx <= 2; dx++) {
        for (sint4 dy = -2; dy <= 2; dy++) {
          if (sq[i + dx + dy * DX] == EMPTY) return false;
        }
      }
    }
  }

  return true;
}


bool AxClientBoard::is_finished() const {
  return false;
}
