/*
    (c) Michael Buro, mic@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "AxClientMove.H"
#include "ClientBoard.H"

// parse ascii-move [e.g. "E13" ]
// return true iff syntax correct

String AxClientMove::str2coords(String &s)
{
  if (s == "") return "empty move string";
  
  vector<String> vec;
  String::parse(s, vec, '-');

  if (vec.size() != 2) {
    return "illegal coords; e.g. a1-a8";
  }
    
  sint4 coords[2];

  FORS (i, 2) {

    sint4 x = toupper(vec[i][0]) - 'A';
    if (x < 0 || x >= ClientBoard::MAX_BOARD_WIDTH) {
      return "sq-x out of range";
    }
    
    const char *p = vec[i].c_str();
    sint4 y;
    
    if (sscanf(&p[1], "%d", &y) != 1) return "missing sq-y";
    
    --y;
    
    if (y < 0 || y >= ClientBoard::MAX_BOARD_WIDTH) {
      return "sq-y out of range";
    }

    coords[i] = ClientBoard::xy2ind(x, y);
  }

  from = coords[0];
  to = coords[1];
  return "";
}

bool AxClientMove::is_valid() const
{
  return from >= 0 && from < ClientBoard::MAX_BOARD_SIZE &&
    to >= 0 && to < ClientBoard::MAX_BOARD_SIZE;
}

String AxClientMove::coords2str() const
{
  if (from == UNDEF) {

    return "?";

  } else {

    return coord2str(from) + "-" + coord2str(to);
  }
}
