/*
    (c) Michael Buro, mic@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "CksClientMove.H"
#include "ClientBoard.H"

// parse ascii-move [e.g. "E13" ]
// return true iff syntax correct

String CksClientMove::str2coords(String &s) {
  
  if (s == "") return "empty move string";
  
  vector<String> vec;
  String::parse(s, vec, '-');

  if (vec.size() < 2) {
    return "illegal coords; e.g. a1-c3";
  }

  ms.erase(ms.begin(), ms.end());

  FORU (i, vec.size()) {

    int x = toupper(vec[i][0]) - 'A';
    if (x < 0 || x >= ClientBoard::MAX_BOARD_WIDTH) {
      return "sq-x out of range";
    }
    
    const char *p = vec[i].c_str();
    int y;
    
    if (sscanf(&p[1], "%d", &y) != 1) {
      return "missing sq-y";
    }
    
    --y;
    
    if (y < 0 || y >= ClientBoard::MAX_BOARD_WIDTH) {
      return "sq-y out of range";
    }

    ms.push_back(ClientBoard::xy2ind(x, y));
  }

  return "";
}


bool CksClientMove::is_valid() const
{
  return ms.size() >= 2;
}

String CksClientMove::coords2str() const
{
  if (ms.size() < 2) {

    return "PA";

  } else {

    String s = "";
    
    FORU (i, ms.size()) {
      s += coord2str(ms[i]);
      if (i < ms.size()-1) s += "-";
    }

    return s;
  }
}
